@@ -186,10 +186,12 @@ KDE_NO_EXPORT Node *RP::Imfl::childFromTag (const String & tag) {
 }
 
 KDE_NO_EXPORT void RP::Imfl::repaint () {
-    if (!active ())
+    if (!active ()) {
         warningLog () << "Spurious Imfl repaint" << endl;
-    else if (surface () && width > 0 && height > 0)
+    } else if (surface () && width > 0 && height > 0) {
+        rp_surface->markDirty ();
         rp_surface->repaint (SRect (0, 0, width, height));
+    }
 }
 
 KDE_NO_CDTOR_EXPORT RP::Image::Image (NodePtr & doc)
