@@ -112,10 +112,10 @@ Control::~Control () {
 void Control::init () {
     unsigned int ver, rel, req, evb, err;
     glob_t gt;
-    bool has_xv = !glob ("/usr/lib/libXv.so*", GLOB_ERR|GLOB_NOSORT, 0, &gt);
+    use_xvideo = !glob ("/usr/lib/libXv.so*", GLOB_ERR|GLOB_NOSORT, 0, &gt);
     globfree (&gt);
-    media_server = new OssoMediaServer (this, m_app->osso_context, has_xv);
-    mplayer = new MPlayer (this, has_xv);
+    media_server = new OssoMediaServer (this, m_app->osso_context, use_xvideo);
+    mplayer = new MPlayer (this, use_xvideo);
 #ifdef __ARMEL__
     npp = new NpPlayer (this);
 #endif
@@ -123,11 +123,12 @@ void Control::init () {
     String p = m_config->readEntry (GCONF_KEY_PLAYER, OssoMediaServer::myname);
     setProcess ((const char *) p);
 
-    if (m_config->readNumEntry (GCONF_KEY_VO, 0)) {
-        mplayer->setUseXv (false);
+    if (use_xvideo)
+        use_xvideo = !m_config->readNumEntry (GCONF_KEY_VO, 0);
+    setUseXVideo (use_xvideo);
+    if (!use_xvideo)
         gtk_check_menu_item_set_active (
                 GTK_CHECK_MENU_ITEM (m_app->menu_item_mplayer_vo), true);
-    }
 
     GtkIconTheme *icon_theme = gtk_icon_theme_get_default();
     addTree (m_app, 0L, //gdk_pixbuf_new_from_xpm_data((const char**)www_xpm));
@@ -255,7 +256,7 @@ void Control::play () {
     }
 }
 
-void Control::stop () {
+void Control::stop (bool keep_fullscreen) {
     debugLog () << "stop" << endl;
     setLoading (100); // no pogress bar
     if (blanking_timer) {
@@ -273,7 +274,8 @@ void Control::stop () {
     m_process->quit ();
     VIEWAREA(m_app)->scheduleRepaint (IRect (0, 0,
                     VIEWAREA(m_app)->width (), VIEWAREA(m_app)->height ()));
-    app_normal_screen (m_app);
+    if (!keep_fullscreen)
+        app_normal_screen (m_app);
     setLength (0);
     setPosition (0);
     updatePlaylistView ();
@@ -287,7 +289,7 @@ String Control::currentMrl () {
 }
 
 void Control::openDocument (NodePtr doc, NodePtr cur) {
-    stop ();
+    stop (true);
     if (playlist_data->document)
         playlist_data->document->document()->dispose ();
     doc->document()->notify_listener = this;
@@ -355,7 +357,6 @@ void Control::playCurrent () {
             updateViewAreaSizes ();
             if (m_process->hasXv ())
                 XSync (GDK_DISPLAY_XDISPLAY (gtk_widget_get_display (VIEWAREA(m_app)->viewWidget ())), false);
-            VIEWAREA(m_app)->setEventFiltering (m_app, true);
             m_process->play (mrl);
         } else
             errorLog() << "playCurrent and mrl not resolved" << endl;
@@ -439,6 +440,7 @@ bool Control::setProcess (const char *pname) {
     gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu), true);
     if (old_process)
         old_process->quit ();
+    VIEWAREA(m_app)->setControlsOverlap (m_process != mplayer || use_xvideo);
 }
 
 bool Control::setProcess (Mrl *mrl) {
@@ -558,6 +560,7 @@ void Control::stateElementChanged (Node* n, Node::State os, Node::State ns) {
     if (ns == Node::state_deactivated &&
             n == playlist_data->document && !back_request) {
         stop ();
+        VIEWAREA(m_app)->setEventFiltering (m_app, false);
         app_set_playing (m_app, NOT_PLAYING);
     } else if ((ns == Node::state_deactivated || ns == Node::state_finished) &&
             n->mrl () && process ()->mrl () == n->mrl()->linkNode ()) {
@@ -574,8 +577,10 @@ void Control::stateElementChanged (Node* n, Node::State os, Node::State ns) {
             m_process->pause ();
         app_set_playing (m_app, ns > Node::state_deferred ? PLAYING : PAUSED);
     } else if (ns == Node::state_activated) {
-        if (n == playlist_data->document)
+        if (n == playlist_data->document) {
+            VIEWAREA(m_app)->setEventFiltering (m_app, true);
             app_set_playing (m_app, PLAYING);
+        }
         if (n->id == SMIL::id_node_body)
             updateViewAreaSizes (); // move away video widget
         if (n->isPlayable () &&
@@ -643,7 +648,6 @@ void Control::stateChanged (Process * p, Process::State os, Process::State ns) {
     if (!p->mrl () && ns > Process::Ready) {
         p->stop (); // reschedule to the NotRunning state
     } else if (ns == Process::NotRunning) {
-        VIEWAREA(m_app)->setEventFiltering (m_app, false);
         if (p == m_process) {
             if (playlist_data->document &&
                     playlist_data->document->state > Node::state_init) {
@@ -825,6 +829,13 @@ void Control::setAspect (float aspect) {
     updateViewAreaSizes ();
 }
 
+void Control::setUseXVideo (bool b) {
+    mplayer->setUseXv (b);
+    use_xvideo = b;
+    if (m_process == mplayer)
+        VIEWAREA(m_app)->setControlsOverlap (b);
+}
+
 void cb_view_area_configure (GtkWidget *widget, GdkEventConfigure *event, Application * app) {
     //debugLog () << "cb_view_area_configure " << event->width<< "x" << event->height << endl;
     CONTROL(app)->updateViewAreaSizes ();
@@ -1088,7 +1099,7 @@ gboolean cb_keyPressed (GtkWidget *, GdkEventKey * e, Application *app) {
             break;
         case GDK_Escape:
             if (app->fullscreen)
-                VIEWAREA(app)->controlsVisible (true);
+                VIEWAREA(app)->controlsVisible (app, true);
             break;
         default:
             KMPlayer::debugLog () << "Keypress " << e->keyval << KMPlayer::endl;
@@ -1125,7 +1136,7 @@ void toggleMediaServer (GtkCheckMenuItem *radio, Application *app) {
 }
 
 void toggleVO (GtkCheckMenuItem *chk, Application *app) {
-        CONTROL(app)->mplayer->setUseXv (!gtk_check_menu_item_get_active (chk));
+    CONTROL(app)->setUseXVideo (!gtk_check_menu_item_get_active (chk));
 }
 
 void app_set_suspend (Application *a, bool suspend) {
