@@ -24,7 +24,6 @@
 #ifdef HAVE_EXPAT
 #include <expat.h>
 #endif
-#include <cairo.h>
 #include "kmplayerplaylist.h"
 #include "kmplayer_asx.h"
 #include "kmplayer_atom.h"
@@ -254,22 +253,9 @@ void Node::appendChild (NodePtr c) {
 }
 
 KDE_NO_EXPORT void Node::insertBefore (NodePtr c, NodePtr b) {
-    if (!b) {
-        appendChild (c);
-    } else {
-        ASSERT (!c->parentNode ());
-        document()->m_tree_version++;
-        if (b->m_prev) {
-            b->m_prev->m_next = c;
-            c->m_prev = b->m_prev;
-        } else {
-            c->m_prev = 0L;
-            m_first_child = c;
-        }
-        b->m_prev = c;
-        c->m_next = b;
-        c->m_parent = this;
-    }
+    ASSERT (!c->parentNode ());
+    document()->m_tree_version++;
+    TreeNode<Node>::insertBefore (c, b);
 }
 
 void Node::removeChild (NodePtr c) {
@@ -431,19 +417,6 @@ String Node::nodeValue () const {
 
 //-----------------------------------------------------------------------------
 
-RefNode::RefNode (NodePtr & d, NodePtr ref)
- : Node (d) {
-    setRefNode (ref);
-}
-
-void RefNode::setRefNode (const NodePtr ref) {
-    ref_node = ref;
-    if (ref_node)
-        tag_name = String (ref_node->nodeName ());
-}
-
-//-----------------------------------------------------------------------------
-
 namespace {
     struct KMPLAYER_NO_EXPORT ParamValue {
         String val;
@@ -766,34 +739,6 @@ void Mrl::parseParam (const TrieString & para, const String & val) {
     }
 }
 
-KDE_NO_CDTOR_EXPORT Surface::Surface (NodePtr n, const SRect & r)
-  : node (n),
-    bounds (r),
-    xscale (1.0),
-    yscale (1.0),
-    background_color (0),
-    dirty (false),
-    surface (0L)
-{}
-
-Surface::~Surface() {
-    if (surface)
-        cairo_surface_destroy (surface);
-}
-
-void Surface::remove () {
-    Surface *sp = parentNode ().ptr ();
-    if (sp) {
-        sp->markDirty ();
-        sp->removeChild (this);
-    }
-}
-
-void Surface::markDirty () {
-    for (Surface *s = this; s; s = s->parentNode ().ptr ())
-        s->dirty = true;
-}
-
 //-----------------------------------------------------------------------------
 
 EventData::EventData (Node *t, Posting *e, EventData *n)
