@@ -741,7 +741,8 @@ GtkWidget * createTreeView (Application * app) {
 }
 
 void cb_playlistMenuActivate (GtkButton * button, Application * app) {
-    bool show_copy = false, show_paste = false, show_cut = false;
+    bool show_copy = false, show_paste = false, show_cut = false,
+         show_title = false;
     GtkTreePath *path;
     PlayListData * pld = getListDataFromSelection (app, &path);
     if (pld) {
@@ -757,12 +758,17 @@ void cb_playlistMenuActivate (GtkButton * button, Application * app) {
                 (!CONTROL(app)->copyPasteUrl.isEmpty () || CONTROL(app)->copyPasteNode);
             show_cut = pld->id == app->playlists_tree_id &&
                 gtk_tree_path_get_depth (path) > 1;
+            show_title = pld->id == app->playlists_tree_id &&
+                (node->id == id_node_playlist_item ||
+                 node->id == id_node_group_node ||
+                 node->id == id_node_html_object);
         }
     }
     gtk_widget_set_sensitive (app->menu_item_playlist_copy, show_copy);
     gtk_widget_set_sensitive (app->menu_item_playlist_paste, show_paste);
     gtk_widget_set_sensitive(app->menu_item_playlist_paste_in_group,show_paste);
     gtk_widget_set_sensitive (app->menu_item_playlist_cut, show_cut);
+    gtk_widget_set_sensitive (app->menu_item_playlist_title, show_title);
 }
 
 void cb_playlistCopy (GtkButton * button, Application * app) {
@@ -796,26 +802,40 @@ void cb_playlistPaste (GtkButton * button, Application * app) {
     }
 }
 
+String getUserInput (Application *app, const gchar *title, const gchar *def) {
+    GtkWidget * dialog= gtk_dialog_new_with_buttons (
+            title,
+            GTK_WINDOW (app->window), GTK_DIALOG_MODAL,
+            GTK_STOCK_OK, GTK_RESPONSE_OK,
+            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
+    GtkWidget *group_edit = gtk_entry_new();
+    if (def)
+        gtk_entry_set_text (GTK_ENTRY (group_edit), def);
+    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox),
+            group_edit);
+    gtk_window_set_default_size(GTK_WINDOW(dialog), 400, 150);
+
+    gtk_widget_show_all (dialog);
+
+    String val;
+    if (GTK_RESPONSE_OK == gtk_dialog_run (GTK_DIALOG (dialog)))
+        val = gtk_entry_get_text (GTK_ENTRY (group_edit));
+
+    gtk_widget_destroy (dialog);
+
+    return val;
+}
+
 void cb_playlistPasteInNewGroup (GtkButton * button, Application * app) {
     GtkTreePath *path;
     PlayListData * pld = getListDataFromSelection (app, &path);
     if (pld) {
         NodePtr node = getNode (pld, path);
         if (node && pld->id == app->playlists_tree_id) {
-            GtkWidget * dialog= gtk_dialog_new_with_buttons (
-                    "Enter Name of the Group",
-                    GTK_WINDOW (app->window), GTK_DIALOG_MODAL,
-                    GTK_STOCK_OK, GTK_RESPONSE_OK,
-                    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
-            GtkWidget * group_edit = gtk_entry_new();
-            gtk_entry_set_text (GTK_ENTRY (group_edit), "New Group");
-            gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox),
-                    group_edit);
-            gtk_window_set_default_size(GTK_WINDOW(dialog), 400, 150);
-            gtk_widget_show_all (dialog);
-            if (GTK_RESPONSE_OK == gtk_dialog_run (GTK_DIALOG (dialog))) {
-                NodePtr newnode = new Group (pld->document, app,
-                        gtk_entry_get_text (GTK_ENTRY (group_edit)));
+            String grp = getUserInput (app,
+                    "Enter Name of the Group", "New Group");
+            if (!grp.isNull ()) {
+                NodePtr newnode = new Group (pld->document, app, grp);
                 newnode->appendChild (CONTROL(app)->copyPasteNode
                         ? CONTROL(app)->copyPasteNode
                         : new PlaylistItem (pld->document, app, false, CONTROL(app)->copyPasteUrl));
@@ -826,7 +846,6 @@ void cb_playlistPasteInNewGroup (GtkButton * button, Application * app) {
                 pld->current = newnode;
                 updateTree (app, app->playlists_tree_id);
             }
-            gtk_widget_destroy (dialog);
         }
     }
 }
@@ -853,6 +872,26 @@ void cb_playlistCut (GtkButton * button, Application * app) {
     }
 }
 
+void cb_playlistTitle (GtkButton *button, Application *app) {
+    GtkTreePath *path;
+    PlayListData *pld = getListDataFromSelection (app, &path);
+    if (pld) {
+        NodePtr node = getNode (pld, path);
+        if (node && pld->id == app->playlists_tree_id) {
+            String title = node->caption;
+            title = getUserInput (app, "Enter new Title", title);
+            if (!title.isNull ()) {
+                static_cast <Element *> (node.ptr ())->setAttribute
+                    (StringPool::attr_title, title);
+                node->caption = title;
+                node->document ()->m_tree_version++;
+                updateTree (app, app->playlists_tree_id);
+            }
+        }
+        gtk_tree_path_free (path);
+    }
+}
+
 void togglePlaylistShow (GtkCheckMenuItem *menuitem, Application * app) {
 }
 
