@@ -242,22 +242,15 @@ KMPLAYER_NO_EXPORT bool MPlayer::play (NodePtr node) {
     wait_for_pause = false;
     setState (Buffering);
     Process::play (node);
-    Mrl *mrl = this->mrl ();
-    if (!mrl) {
-        errorLog() << "Not a Mrl " << (node ? node->nodeName() :"null") << endl;
-        setState (Ready);
-        return false;
-    }
-    String strurl = mrl->absolutePath ();
-    if (strurl.isEmpty ()) {
+    if (m_url.isEmpty ()) {
         setState (Ready);
         return false;
     }
     request_seek = NO_SEEK;
     String wd ("/home/user");
-    URL url (strurl);
+    URL url (m_url);
     if (url.isLocalFile ()) {
-        strurl = url.path ();
+        m_url = url.path ();
         wd = url.path ();
         int pos = wd.lastIndexOf (String ("/"));
         if (pos > -1)
@@ -271,7 +264,9 @@ KMPLAYER_NO_EXPORT bool MPlayer::play (NodePtr node) {
     //    snprintf (buf, sizeof(buf), "xv -wid %u ", m_viewer);
     //else
         snprintf (buf, sizeof(buf),
-                "xv,nokia770:fb_overlay_only:x=%d:y=%d:w=%d:h=%d",
+                has_xv
+                ? "xv,nokia770:fb_overlay_only:x=%d:y=%d:w=%d:h=%d"
+                : "omapfb,xv,nokia770:fb_overlay_only:x=%d:y=%d:w=%d:h=%d",
                 m_rect.x() - (m_rect.x() % 2),
                 m_rect.y() - (m_rect.y() % 2),
                 m_rect.width() - (m_rect.width() % 4),
@@ -279,10 +274,11 @@ KMPLAYER_NO_EXPORT bool MPlayer::play (NodePtr node) {
     cmdline += String (buf);
     snprintf (buf, sizeof(buf), " -wid %u ", m_viewer);
     cmdline += String (buf);
+    Mrl *mrl = this->mrl ();
     if (mrl->repeat > 0)
         cmdline += String (" -loop ") + String::number (mrl->repeat+1);
     cmdline += Char (' ');
-    char *quoted = g_shell_quote ((const char *) strurl);
+    char *quoted = g_shell_quote ((const char *) m_url);
     cmdline += String (quoted);
     g_free (quoted);
     debugLog () << cmdline << endl;
@@ -983,7 +979,7 @@ KMPLAYER_NO_EXPORT bool OssoMediaServer::retry () {
         return false;
     } else if (retry_timer) {
         return true;
-    } else if (m_mrl && !mrl ()->audio_only && mrl ()->actor) {
+    } else if (m_mrl && !mrl ()->audio_only && mrl ()->media_info) {
         try_audio_only = true;
         retry_timer = g_timeout_add (100, cb_retryAudioOnly, this);
         return true;
@@ -1029,17 +1025,9 @@ KMPLAYER_NO_EXPORT bool OssoMediaServer::play (NodePtr node) {
     else
         try_audio_only = false;
     Process::play (node);
-    Mrl *mrl = this->mrl ();
-    if (!mrl)
-        return false;
-    String abspath = mrl->absolutePath ();
     stream_name.clear ();
-    if (abspath.isEmpty ()) {
-        errorLog () << "empty path " << mrl->nodeName () << " " << mrl->src << endl;
-        if (!mrl->src.isEmpty ()) // bug in absolutePath
-            abspath = mrl->src;
-        else
-            return false;
+    if (m_url.isEmpty ()) {
+        return false;
     }
     have_played_stream = false;
     DBusConnection * connection = (DBusConnection *)osso_get_dbus_connection (m_osso_context);
@@ -1062,13 +1050,13 @@ KMPLAYER_NO_EXPORT bool OssoMediaServer::play (NodePtr node) {
         }
     }
     volume (0, false);
-    no_video = mrl->audio_only || try_audio_only;
+    no_video = mrl ()->audio_only || try_audio_only;
     current_service = no_video
         ? media_server_imusic
         : media_server_ivideo;
     DBusMessage *msg = dbus_message_new_method_call (0L, media_server_path, current_service, "play_media");
     dbus_message_set_destination (msg, media_server);
-    const char * path = (const char *) abspath;
+    const char * path = (const char *) m_url;
     debugLog () << "OssoMediaServer::play " << current_service << " " << path << endl;
     dbus_message_append_args(msg, DBUS_TYPE_STRING,&path,DBUS_TYPE_INVALID);
     //dbus_connection_send (connection, message, NULL);
@@ -1349,20 +1337,14 @@ KMPLAYER_NO_EXPORT bool NpPlayer::play (NodePtr node) {
         return true;
     setState (Buffering);
     Process::play (node);
-    Mrl *mrl = this->mrl ();
-    if (!mrl) {
-        errorLog() << "Not a Mrl " << (node ? node->nodeName() :"null") << endl;
-        setState (Ready);
-        return false;
-    }
-    String strurl = mrl->absolutePath ();
-    if (strurl.isEmpty ()) {
+    if (m_url.isEmpty ()) {
         setState (Ready);
         return false;
     }
     streams = g_tree_new (streamCompare);
 
     GTree *params = g_tree_new ((GCompareFunc)::strcmp);
+    Mrl *mrl = this->mrl ();
     if (mrl->id == id_node_html_object) {
         for (Node *n = mrl->firstChild().ptr(); n; n = n->nextSibling().ptr()) {
             /*if (n->id == KMPlayer::id_node_param) {
@@ -1385,7 +1367,7 @@ KMPLAYER_NO_EXPORT bool NpPlayer::play (NodePtr node) {
             }
         }
     } else {
-        g_tree_insert (params, strdup ("SRC"), strdup ((const char *) strurl));
+        g_tree_insert (params, strdup ("SRC"), strdup ((const char *) m_url));
     }
     argc = g_tree_nnodes (params);
     argn = (char **) malloc (argc * sizeof (char *));
@@ -1402,7 +1384,7 @@ KMPLAYER_NO_EXPORT bool NpPlayer::play (NodePtr node) {
         warningLog() << "failed to create new instance" << endl;
         return false;
     }
-    nppInstanceStart ((const char *)strurl, mime, npdata);
+    nppInstanceStart ((const char *)m_url, mime, npdata);
     return true;
 }
 
