@@ -41,82 +41,18 @@ using namespace KMPlayer;
 
 //-------------------------------------------------------------------------
 
-namespace {
-/**                                   a  b  0
- * Matrix for coordinate transforms   c  d  0
- *                                    tx ty 1     */
-class KMPLAYER_NO_EXPORT Matrix {
-    float a, b, c, d;
-    Single tx, ty;
-public:
-    Matrix ();
-    Matrix (const Matrix & matrix);
-    Matrix (Single xoff, Single yoff, float xscale, float yscale);
-    void getXY (Single & x, Single & y) const;
-    void getXYWH (Single & x, Single & y, Single & w, Single & h) const;
-    void invXYWH (Single & x, Single & y, Single & w, Single & h) const;
-    void transform (const Matrix & matrix);
-    void scale (float sx, float sy);
-    void translate (Single x, Single y);
-    // void rotate (float phi); // add this when needed
-};
-
+static void clearSurface (cairo_t *cr, const IRect &rect) {
+    cairo_save (cr);
+    cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
+    cairo_rectangle (cr, rect.x, rect.y, rect.w, rect.h);
+    cairo_fill (cr);
+    cairo_restore (cr);
 }
 
-Matrix::Matrix () : a (1.0), b (0.0), c (0.0), d (1.0), tx (0), ty (0) {}
-
-Matrix::Matrix (const Matrix & m)
- : a (m.a), b (m.b), c (m.c), d (m.d), tx (m.tx), ty (m.ty) {}
-
-Matrix::Matrix (Single xoff, Single yoff, float xscale, float yscale)
- : a (xscale), b (0.0), c (0.0), d (yscale), tx (xoff), ty (yoff) {}
-
-void Matrix::getXY (Single & x, Single & y) const {
-    x = Single (x * a) + tx;
-    y = Single (y * d) + ty;
-}
-
-void Matrix::getXYWH (Single & x, Single & y, Single & w, Single & h) const {
-    getXY (x, y);
-    w *= a;
-    h *= d;
-}
-
-void Matrix::invXYWH (Single & x, Single & y, Single & w, Single & h) const {
-    if (a > 0.00001 && d > 0.00001) {
-        w /= a;
-        h /= d;
-        x = Single ((x - tx) / a);
-        y = Single ((y - ty) / d);
-    } else {
-        warningLog () << "Not invering " << a << ", " << d << " scale" << endl;
-    }
-}
-
-void Matrix::transform (const Matrix & matrix) {
-    // TODO: rotate
-    a *= matrix.a;
-    d *= matrix.d;
-    tx = Single (tx * matrix.a) + matrix.tx;
-    ty = Single (ty * matrix.d) + matrix.ty;
-}
-
-void Matrix::scale (float sx, float sy) {
-    a *= sx;
-    d *= sy;
-    tx *= sx;
-    ty *= sy;
-}
-
-void Matrix::translate (Single x, Single y) {
-    tx += x;
-    ty += y;
-}
-
-//-----------------------------------------------------------------------------
-
-void ImageData::copyImage (Surface *s, int w, int h, cairo_surface_t *similar) {
+void ImageData::copyImage (Surface *s, int w, int h, cairo_surface_t *similar,
+        unsigned int *penalty, CalculatedSizer *zoom) {
     cairo_surface_t *src_sf;
+    bool clear = false;
 
     unsigned char *bits = NULL;
     if (surface) {
@@ -135,6 +71,7 @@ void ImageData::copyImage (Surface *s, int w, int h, cairo_surface_t *similar) {
                     width, height);
             cairo_pattern_t *pat = cairo_pattern_create_for_surface (src_sf);
             cairo_pattern_set_extend (pat, CAIRO_EXTEND_NONE);
+            cairo_pattern_set_filter (pat, CAIRO_FILTER_FAST);
             cairo_t *cr = cairo_create (surface);
             cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
             cairo_set_source (cr, pat);
@@ -142,24 +79,40 @@ void ImageData::copyImage (Surface *s, int w, int h, cairo_surface_t *similar) {
             cairo_destroy (cr);
             cairo_pattern_destroy (pat);
             cairo_surface_destroy (src_sf);
+            cairo_surface_flush (surface);
             g_free (bits);
             src_sf = surface;
             delete image;
             image = NULL;
+            *penalty += width * height / (10 * 480);
         }
     }
 
     cairo_pattern_t *img_pat = cairo_pattern_create_for_surface (src_sf);
     cairo_pattern_set_extend (img_pat, CAIRO_EXTEND_NONE);
-    if (w != width && h != height) {
+    if (zoom) {
+        cairo_matrix_t mat;
+        Single zx, zy, zw, zh;
+        zoom->calcSizes (NULL, width, height, zx, zy, zw, zh);
+        cairo_matrix_init_translate (&mat, zx, zy);
+        cairo_matrix_scale (&mat, 1.0 * zw/w, 1.0 * zh/h);
+        cairo_pattern_set_matrix (img_pat, &mat);
+        *penalty += w * h / (1 * 480);
+    } else if (w != width && h != height) {
         cairo_matrix_t mat;
         cairo_matrix_init_scale (&mat, 1.0 * width/w, 1.0 * height/h);
         cairo_pattern_set_matrix (img_pat, &mat);
+        *penalty += w * h / (1 * 480);
     }
     if (!s->surface)
         s->surface = cairo_surface_create_similar (similar,
-                has_alpha ?  CAIRO_CONTENT_COLOR_ALPHA : CAIRO_CONTENT_COLOR, w, h);
+                has_alpha ?
+                CAIRO_CONTENT_COLOR_ALPHA : CAIRO_CONTENT_COLOR, w, h);
+    else
+        clear = true;
     cairo_t *cr = cairo_create (s->surface);
+    if (clear)
+        clearSurface (cr, IRect (0, 0, w, h));
     cairo_set_source (cr, img_pat);
     if (!has_alpha)
         cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
@@ -175,98 +128,7 @@ void ImageData::copyImage (Surface *s, int w, int h, cairo_surface_t *similar) {
 
 //-------------------------------------------------------------------------
 
-namespace KMPlayer {
-
-class KMPLAYER_NO_EXPORT ViewSurface : public Surface {
-public:
-    ViewSurface (ViewArea * widget);
-    ViewSurface (ViewArea * widget, NodePtr owner, const SRect & rect);
-    ~ViewSurface ();
-
-    void clear () { m_first_child = 0L; }
-
-    SurfacePtr createSurface (NodePtr owner, const SRect & rect);
-    IRect toScreen (Single x, Single y, Single w, Single h);
-    IRect clipToScreen (Single x, Single y, Single w, Single h);
-    void resize (const SRect & rect);
-    void repaint ();
-    void repaint (const SRect &rect);
-
-    NodePtrW current_video;
-    ViewArea *view_widget;
-};
-
-} // namespace
-
-KDE_NO_CDTOR_EXPORT ViewSurface::ViewSurface (ViewArea * widget)
-  : Surface (NULL, SRect (0, 0, 342, 260)), //widget->width(),widget->height()))
-    view_widget (widget)
-{}
-
-KDE_NO_CDTOR_EXPORT
-ViewSurface::ViewSurface (ViewArea * widget, NodePtr owner, const SRect & rect)
-  : Surface (owner, rect), view_widget (widget) {}
-
-KDE_NO_CDTOR_EXPORT ViewSurface::~ViewSurface() {
-    //debugLog () << "~ViewSurface" << endl;
-}
-
-KDE_NO_EXPORT
-SurfacePtr ViewSurface::createSurface (NodePtr owner, const SRect & rect) {
-    SurfacePtr surface = new ViewSurface (view_widget, owner, rect);
-    appendChild (surface);
-    return surface;
-}
-
-KDE_NO_EXPORT void ViewSurface::resize (const SRect &r) {
-    bounds = r;
-    /*if (rect == nrect)
-        ;//return;
-    SRect pr = rect.unite (nrect); // for repaint
-    rect = nrect;*/
-}
-
-KDE_NO_EXPORT IRect ViewSurface::toScreen (Single x, Single y, Single w, Single h) {
-    Matrix matrix (0, 0, xscale, yscale);
-    matrix.translate (bounds.x (), bounds.y ());
-    for (SurfacePtr s = parentNode(); s; s = s->parentNode()) {
-        matrix.transform(Matrix (0, 0, s->xscale, s->yscale));
-        matrix.translate (s->bounds.x (), s->bounds.y ());
-    }
-    matrix.getXYWH (x, y, w, h);
-    return IRect (x, y, w, h);
-}
-
-KDE_NO_EXPORT IRect ViewSurface::clipToScreen (Single x, Single y, Single w, Single h) {
-    Matrix m (0, 0, xscale, yscale);
-    m.translate (bounds.x (), bounds.y ());
-    m.getXYWH (x, y, w, h);
-    SRect r = bounds.intersect (SRect (x, y, w, h));
-    x= r.x (), y = r.y (), w = r.width (), h = r.height ();
-    for (SurfacePtr s = parentNode (); s; s = s->parentNode ()) {
-        m = Matrix (0, 0, s->xscale, s->yscale);
-        m.translate (s->bounds.x (), s->bounds.y ());
-        m.getXYWH (x, y, w, h);
-        r = SRect (s->bounds).intersect (SRect (x, y, w, h));
-        x= r.x (); y = r.y (); w = r.width (); h = r.height ();
-    }
-    return IRect (x, y, w, h);
-}
-
-KDE_NO_EXPORT void ViewSurface::repaint (const SRect &r) {
-    markDirty ();
-    view_widget->scheduleRepaint (clipToScreen (r.x (), r.y (), r.width (), r.height ()));
-    //kdDebug() << "Surface::repaint x:" << (int)x << " y:" << (int)y << " w:" << (int)w << " h:" << (int)h << endl;
-}
-
-KDE_NO_EXPORT void ViewSurface::repaint () {
-    markDirty ();
-    view_widget->scheduleRepaint (clipToScreen (0, 0, bounds.width (), bounds.height ()));
-}
-
-//-------------------------------------------------------------------------
-
-#define CAIRO_SET_SOURCE_RGB(cr,c)           \
+#define CAIRO_SET_SOURCE_RGB(cr,c)            \
     cairo_set_source_rgb ((cr),               \
             1.0 * (((c) >> 16) & 0xff) / 255, \
             1.0 * (((c) >> 8) & 0xff) / 255,  \
@@ -285,13 +147,14 @@ class KMPLAYER_NO_EXPORT CairoPaintVisitor : public Visitor {
     cairo_pattern_t * cur_pat;
     cairo_matrix_t cur_mat;
     float opacity;
+public:
+    unsigned int penalty;
     bool toplevel;
 
-    void traverseRegion (Node *reg);
+    void traverseRegion (Node *reg, Surface *s);
     void updateExternal (SMIL::MediaType *av, SurfacePtr s);
     void paint(SMIL::MediaType *, Surface *, int x, int y, const IRect &);
     void video (Mrl *mt, Surface *s);
-public:
     cairo_t * cr;
     CairoPaintVisitor (ViewArea *v, cairo_surface_t * cs, Matrix m,
             const IRect & rect, bool toplevel=false);
@@ -340,6 +203,7 @@ CairoPaintVisitor::CairoPaintVisitor (ViewArea *v, cairo_surface_t *cs,
  cairo_surface (cs), matrix (m), toplevel (top) {
     cr = cairo_create (cs);
     if (toplevel) {
+        penalty = rect.w * rect.h / (20 * 480);
         cairo_rectangle (cr, rect.x, rect.y, rect.w, rect.h);
         cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
         cairo_set_tolerance (cr, 0.5 );
@@ -350,11 +214,8 @@ CairoPaintVisitor::CairoPaintVisitor (ViewArea *v, cairo_surface_t *cs,
         cairo_fill (cr);
 #endif
     } else {
-        cairo_save (cr);
-        cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
-        cairo_rectangle (cr, rect.x, rect.y, rect.w, rect.h);
-        cairo_fill (cr);
-        cairo_restore (cr);
+        penalty = 0;
+        clearSurface (cr, rect);
     }
 }
 
@@ -387,7 +248,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Smil *s) {
         s->layout_node->accept (this);
 }
 
-KDE_NO_EXPORT void CairoPaintVisitor::traverseRegion (Node *node) {
+KDE_NO_EXPORT void CairoPaintVisitor::traverseRegion (Node *node, Surface *s) {
     // next visit receivers
     NodeRefListPtr nl = nodeMessageReceivers (node, MsgSurfaceAttach);
     if (nl) {
@@ -395,29 +256,27 @@ KDE_NO_EXPORT void CairoPaintVisitor::traverseRegion (Node *node) {
             if (c->data)
                 c->data->accept (this);
     }
-    // finally visit children, accounting for z-order FIXME optimize
-    NodeRefList sorted;
-    for (NodePtr n = node->firstChild (); n; n = n->nextSibling ()) {
-        if (n->id != SMIL::id_node_region)
-            continue;
-        SMIL::Region * r = static_cast <SMIL::Region *> (n.ptr ());
-        NodeRefItemPtr rn = sorted.first ();
-        for (; rn; rn = rn->nextSibling ())
-            if (r->z_order < convertNode <SMIL::Region> (rn->data)->z_order) {
-                sorted.insertBefore (new NodeRefItem (n), rn);
-                break;
-            }
-        if (!rn)
-            sorted.append (new NodeRefItem (n));
+    /*for (SurfacePtr c = s->lastChild (); c; c = c->previousSibling ()) {
+        if (c->node && c->node->id != SMIL::id_node_region &&
+        c->node && c->node->id != SMIL::id_node_root_layout)
+            c->node->accept (this);
+        else
+            break;
+    }*/
+    // finally visit region children
+    for (SurfacePtr c = s->firstChild (); c; c = c->nextSibling ()) {
+        if (c->node && c->node->id == SMIL::id_node_region)
+            c->node->accept (this);
+        else
+            break;
     }
-    for (NodeRefItemPtr r = sorted.first (); r; r = r->nextSibling ())
-        r->data->accept (this);
+    s->dirty = false;
 }
 
-KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Layout *reg) {
-    //debugLog() << "Visit " << reg->nodeName() << endl;
-    if (reg->root_layout) {
-        Surface *s = (Surface *)reg->root_layout->message (MsgQueryRoleDisplay);
+KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Layout *layout) {
+    //debugLog() << "Visit " << layout->nodeName() << endl;
+    if (layout->root_layout) {
+        Surface *s = (Surface *)layout->root_layout->message (MsgQueryRoleDisplay);
         if (s) {
             //cairo_save (cr);
             Matrix m = matrix;
@@ -437,8 +296,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Layout *reg) {
 
             matrix = Matrix (0, 0, s->xscale, s->yscale);
             matrix.transform (m);
-            traverseRegion (reg->root_layout);
-            traverseRegion (reg);
+            traverseRegion (layout->root_layout, s);
             //cairo_restore (cr);
             matrix = m;
             clip = clip_save;
@@ -480,7 +338,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::RegionBase *reg) {
                 Single h = bg_img->height;
                 matrix.getXYWH (x1, y1, w, h);
                 if (!s->surface)
-                    bg_img->copyImage (s, w, h, cairo_surface);
+                    bg_img->copyImage (s, w, h, cairo_surface, &penalty);
                 if (bg_img->has_alpha)
                     cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
                 cairo_pattern_t *pat = cairo_pattern_create_for_surface (s->surface);
@@ -497,10 +355,11 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::RegionBase *reg) {
             }
             cairo_restore (cr);
         }
-        traverseRegion (reg);
+        traverseRegion (reg, s);
         cairo_restore (cr);
         matrix = m;
         clip = clip_save;
+        s->dirty = false;
     }
 }
 
@@ -690,6 +549,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::paint (SMIL::MediaType *mt, Surface *s,
     if (mt->active_trans) {
         IRect clip_save = clip;
         clip = rect;
+        penalty += rect.w * rect.h / (20 * 480);
         cur_media = mt;
         mt->active_trans->accept (this);
         clip = clip_save;
@@ -702,6 +562,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::paint (SMIL::MediaType *mt, Surface *s,
     }
     opacity *= mt->opacity / 100.0;
     if (opacity < 0.99) {
+        penalty += rect.w * rect.h / (40 * 480);
         cairo_operator_t op = cairo_get_operator (cr);
         cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
         cairo_clip (cr);
@@ -740,7 +601,7 @@ void CairoPaintVisitor::updateExternal (SMIL::MediaType *av, SurfacePtr s) {
     bool rp_or_smil = false;
     Mrl *ext_mrl = findActiveMrl (av->external_tree.ptr (), &rp_or_smil);
     if (!ext_mrl)
-         return;
+        return;
     if (!rp_or_smil) {
         video (ext_mrl, s.ptr ());
         return;
@@ -766,6 +627,7 @@ void CairoPaintVisitor::updateExternal (SMIL::MediaType *av, SurfacePtr s) {
         }
         CairoPaintVisitor visitor (view_widget, s->surface, m, r);
         ext_mrl->accept (&visitor);
+        penalty += visitor.penalty;
         s->dirty = false;
     }
     paint (av, s.ptr (), x, y, clip_rect);
@@ -800,7 +662,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::ImageMediaType * img) {
     if (clip_rect.isEmpty ())
         return;
     if (!s->surface || s->dirty)
-        id->copyImage (s, w, h, cairo_surface);
+        id->copyImage (s, w, h, cairo_surface, &penalty, img->pan_zoom);
     if (id->has_alpha)
         cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
     paint (img, s, x, y, clip_rect);
@@ -1062,7 +924,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::Fadein * fi) {
             if ((int)fi->w && (int)fi->h && (int)sw && (int)sh) {
                 if (!img->img_surface->surface)
                     im->cached_img->copyImage (img->img_surface,
-                            img->width, img->height, cairo_surface);
+                            img->width, img->height, cairo_surface, &penalty);
                 cairo_matrix_t matrix;
                 cairo_matrix_init_identity (&matrix);
                 float scalex = 1.0 * sw / fi->w;
@@ -1115,7 +977,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::Crossfade * cf) {
             if ((int)cf->w && (int)cf->h && (int)sw && (int)sh) {
                 if (!img->img_surface->surface)
                     im->cached_img->copyImage (img->img_surface,
-                            img->width, img->height, cairo_surface);
+                            img->width, img->height, cairo_surface, &penalty);
                 cairo_save (cr);
                 cairo_matrix_t matrix;
                 cairo_matrix_init_identity (&matrix);
@@ -1177,7 +1039,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::Wipe * wipe) {
             if ((int)w && (int)h) {
                 if (!img->img_surface->surface)
                     im->cached_img->copyImage (img->img_surface,
-                            img->width, img->height, cairo_surface);
+                            img->width, img->height, cairo_surface, &penalty);
                 cairo_matrix_t matrix;
                 cairo_matrix_init_identity (&matrix);
                 float scalex = 1.0 * sw / wipe->w;
@@ -1501,7 +1363,7 @@ public:
                 //cairo_xlib_surface_set_size (s->surface, w, h);
             Window wid = gdk_x11_drawable_get_xid (
                     GDK_DRAWABLE (m_view_area->viewWidget ()->window));
-            XFreePixmap (display (), backing_store);
+            destroyBackingStore ();
             backing_store = XCreatePixmap (dpy, wid, w, h,
                     gdk_drawable_get_depth (
                         GDK_DRAWABLE (m_view_area->viewWidget ()->window)));
@@ -1571,11 +1433,15 @@ KDE_NO_CDTOR_EXPORT ViewArea::ViewArea ()
    aspect (0.0),
    widget (0L),
    video (0L),
-   surface (new ViewSurface (this)),
+   surface (new Surface (this)),
+   last_repaint_time (0),
+   next_repaint_time (0),
+   penalty (0),
    m_repaint_timer (0),
    m_fullscreen (false),
    m_event_filter_on (false),
-   m_updaters_enabled (true) {
+   m_updaters_enabled (true),
+   m_controls_overlap (true) {
 }
 
 KDE_NO_CDTOR_EXPORT ViewArea::~ViewArea () {
@@ -1623,6 +1489,7 @@ KDE_NO_EXPORT void ViewArea::syncVisual (const IRect & rect) {
                 IRect (ex, ey, ew, eh), true);
         if (surface->node)
             surface->node->accept (&visitor);
+        penalty = visitor.penalty;
     }
     cairo_surface_flush (surface->surface);
     d->swapBuffer (ex, ey, ew, eh, ex, ey);
@@ -1636,7 +1503,6 @@ KDE_NO_EXPORT void ViewArea::paintEvent (const IRect & rect) {
 
 KDE_NO_EXPORT void ViewArea::updateSurfaceBounds () {
     Single x, y, w = width (), h = height ();
-    surface->resize (SRect (x, y, w, h));
     Mrl *mrl = surface->node ? surface->node->mrl () : NULL;
     if (w > 0 && h > 0 &&
             mrl && mrl->width > 0 && mrl->height > 0) {
@@ -1657,7 +1523,12 @@ KDE_NO_EXPORT void ViewArea::updateSurfaceBounds () {
         surface->xscale = 1.0;
         surface->yscale = 1.0;
     }
-    surface->bounds = SRect (x, y, w, h);
+    if (surface->node) {
+        surface->bounds = SRect (x, y, w, h);
+        surface->node->message (MsgSurfaceBoundsUpdate, (void *) true);
+    } else {
+        surface->resize (SRect (x, y, w, h), true);
+    }
     scheduleRepaint (IRect (0, 0, width (), height ()));
 }
 
@@ -1667,13 +1538,12 @@ KDE_NO_EXPORT void ViewArea::resizeEvent () {
     Single x, y, w = width (), h = height ();
     if (w <= 0 || h <= 0)
         return;
-    if (surface->node) {
-        NodePtr n = surface->node;
-        surface = new ViewSurface (this);
-        surface->node = n;
-    }
+    if (surface->node)
+        d->destroyBackingStore ();
     updateSurfaceBounds ();
     d->resizeSurface (surface.ptr ());
+    if (controls_timer && !m_controls_overlap)
+        h -= 60;
     if (video_node && Mrl::SingleMode == video_node->mrl ()->view_mode)
         setAudioVideoGeometry (IRect (x, y, w, h), 0L);
 }
@@ -1736,7 +1606,7 @@ void ViewArea::setAspect (float a) {
 }
 
 KDE_NO_EXPORT Surface *ViewArea::getSurface (Mrl *mrl) {
-    static_cast <ViewSurface *> (surface.ptr ())->clear ();
+    surface->clear ();
     surface->node = mrl;
     //m_view->viewer()->resetBackgroundColor ();
     if (mrl) {
@@ -1756,14 +1626,18 @@ KDE_NO_EXPORT void ViewArea::scheduleRepaint (const IRect &rect) {
         m_repaint_rect = m_repaint_rect.unite (rect);
     else {
         m_repaint_rect = rect;
-        m_repaint_timer = g_timeout_add (50, cb_scheduledPaint, this);
+        scheduleRepaintTimeout (surface->node
+                ? surface->node->document ()->last_event_time
+                : 0);
     }
 }
 
 KDE_NO_EXPORT void ViewArea::addUpdater (Node *node) {
     m_updaters = new RepaintUpdater (node, m_updaters);
     if (m_updaters_enabled && !m_repaint_timer)
-        m_repaint_timer = g_timeout_add (50, cb_scheduledPaint, this);
+        scheduleRepaintTimeout (surface->node
+                ? surface->node->document ()->last_event_time
+                : 0);
 }
 
 KDE_NO_EXPORT void ViewArea::removeUpdater (Node *node) {
@@ -1814,7 +1688,7 @@ void ViewArea::enableUpdaters (bool enable, unsigned int skip) {
         UpdateEvent event (m_updaters->node->document (), skip);
         propagateUpdatersEvent (m_updaters, &event);
         if (!m_repaint_timer)
-            m_repaint_timer = g_timeout_add (50, cb_scheduledPaint, this);;
+            scheduleRepaintTimeout (event.cur_event_time);
     } else if (!enable && m_repaint_timer && m_repaint_rect.isEmpty ()) {
         g_source_remove (m_repaint_timer);
         m_repaint_timer = 0;
@@ -1824,18 +1698,35 @@ void ViewArea::enableUpdaters (bool enable, unsigned int skip) {
 KDE_NO_EXPORT bool ViewArea::timeoutPaintEvent () {
     m_updaters = getFirstUpdater (m_updaters);
     if (m_updaters_enabled && m_updaters) {
-            UpdateEvent event (m_updaters->node->document (), 0);
-            propagateUpdatersEvent (m_updaters, &event);
+        UpdateEvent event (m_updaters->node->document (), 0);
+        next_repaint_time = event.cur_event_time;
+        propagateUpdatersEvent (m_updaters, &event);
     }
     if (!m_repaint_rect.isEmpty ())
         syncVisual (m_repaint_rect);
+    last_repaint_time = next_repaint_time;
     if (m_updaters_enabled && m_updaters)
-        return true;
-
+        return scheduleRepaintTimeout (last_repaint_time);
     m_repaint_timer = 0;
     return false;
 }
 
+bool ViewArea::scheduleRepaintTimeout (unsigned int cur_time) {
+    int timeout;
+    if (!cur_time)
+        timeout = 20;
+    else if (last_repaint_time + penalty < cur_time)
+        timeout = 20;
+    else
+        timeout = 20 + penalty - (cur_time - last_repaint_time);
+    next_repaint_time = cur_time + timeout;
+    if (m_repaint_timer && abs (cur_timeout - timeout) < 5)
+        return true;
+    cur_timeout = timeout;
+    m_repaint_timer = g_timeout_add (cur_timeout, cb_scheduledPaint, this);;
+    return false;
+}
+
 KDE_NO_EXPORT int ViewArea::width () const {
     int w = 0, h;
     if (widget)
@@ -1896,6 +1787,8 @@ static
 GdkFilterReturn videoWidgetFilter (GdkXEvent *xe, GdkEvent *e, Application *a) {
     XEvent *event = (XEvent *) xe;
     ViewArea *view_area = VIEWAREA(a);
+    static Time down_time;
+    static int down_x;
 
     if (view_area->controls &&
             event->xany.window == GDK_DRAWABLE_XID (view_area->controls))
@@ -1930,15 +1823,34 @@ GdkFilterReturn videoWidgetFilter (GdkXEvent *xe, GdkEvent *e, Application *a) {
             }
             break;
         }
-        case ButtonPress:
+        case ButtonPress: {
+            Window v = GDK_DRAWABLE_XID(GDK_DRAWABLE(view_area->videoWindow()));
+            Window w = GDK_DRAWABLE_XID (GDK_DRAWABLE (view_area->viewWidget ()->window));
+            Rect vr = view_area->videoGeometry ();
             if (a->fullscreen &&
-                    event->xany.window == GDK_DRAWABLE_XID (GDK_DRAWABLE (view_area->viewWidget ()->window)) &&
-                    event->xbutton.x > 720 && event->xbutton.y < 60) {
+                    (event->xany.window == w &&
+                     event->xbutton.x > 720 && event->xbutton.y < 80) ||
+                    (event->xany.window == v &&
+                     event->xbutton.x + vr.x () > 720 &&
+                     event->xbutton.y + vr.y () < 80)) {
                 if (!view_area->controls_timer)
                     updatePlayTree (a);
-                view_area->controlsVisible (true);
+                view_area->controlsVisible (a, true);
+            } else if (event->xany.window == w || event->xany.window == v) {
+                if (event->xbutton.time - down_time < 1000 &&
+                        abs (down_x - event->xbutton.x) < 80 ) {
+                    if (a->fullscreen)
+                        app_normal_screen (a);
+                    else
+                        app_full_screen (a);
+                    down_time = 0;
+                } else {
+                    down_x = event->xbutton.x;
+                    down_time = event->xbutton.time;
+                }
             }
             break;
+        }
     }
     return GDK_FILTER_CONTINUE;
 }
@@ -1976,7 +1888,7 @@ GdkFilterReturn controlsFilter (GdkXEvent *xe, GdkEvent *e, Application *app) {
             }
             if (w > 0) {
                 gdk_window_clear_area_e (VIEWAREA(app)->controls, x, 0, w, 60);
-                view_area->controlsVisible (true);
+                view_area->controlsVisible (app, true);
             }
         }
         break;
@@ -1987,7 +1899,7 @@ GdkFilterReturn controlsFilter (GdkXEvent *xe, GdkEvent *e, Application *app) {
                 gdk_window_clear_area_e (VIEWAREA(app)->controls, 110, 0, 545, 60);
                 rowSelected (app);
             } else {
-                view_area->controlsVisible (false);
+                view_area->controlsVisible (app, false);
             }
         }
         break;
@@ -2081,23 +1993,30 @@ KDE_NO_EXPORT void ViewArea::setOutputWidget (Application *app, GtkWidget * w) {
     //resizeEvent ();
 }
 
-gboolean cb_controlsTimeout (ViewArea *view_area) {
-    view_area->controls_timer = 0;
-    view_area->controlsVisible (false);
+static gboolean cb_controlsTimeout (Application *app) {
+    VIEWAREA(app)->controls_timer = 0;
+    VIEWAREA(app)->controlsVisible (app, false);
     return false;
 }
 
-void ViewArea::controlsVisible (bool show) {
+void ViewArea::controlsVisible (Application *app, bool show) {
     if (controls_timer)
         g_source_remove (controls_timer);
     if (show) {
-        if (!controls_timer)
+        bool update_sizes = false;
+        if (!controls_timer) {
             gdk_window_move (controls, 0, 420);
-        controls_timer = g_timeout_add (10000, (GSourceFunc) cb_controlsTimeout, this);
+            update_sizes = true;
+        }
+        controls_timer = g_timeout_add (10000, (GSourceFunc) cb_controlsTimeout, app);
+        if (update_sizes)
+            CONTROL(app)->updateViewAreaSizes ();
     } else {
-        if (controls)
-            gdk_window_move (controls, 0, 490);
         controls_timer = 0;
+        if (controls) {
+            gdk_window_move (controls, 0, 490);
+            CONTROL(app)->updateViewAreaSizes ();
+        }
     }
 }
 
@@ -2207,7 +2126,7 @@ gboolean cbViewAreaFullscreenMapped (GtkWidget *w, GdkEvent *, Application *a) {
     } else {
         gdk_window_raise (view_area->controls);
     }
-    view_area->controlsVisible (false);
+    view_area->controlsVisible (a, false);
     cb_view_area_configure (w, 0L, a);
     return false;
 }
@@ -2215,7 +2134,7 @@ gboolean cbViewAreaFullscreenMapped (GtkWidget *w, GdkEvent *, Application *a) {
 void cbViewAreaFullscreenUnmapped (GtkWidget *w, Application *a) {
     ViewArea *view_area = VIEWAREA(a);
     view_area->setOutputWidget (a, a->view_area);
-    view_area->controlsVisible (false);
+    view_area->controlsVisible (a, false);
     cb_view_area_configure (w, 0L, a);
 }
 
@@ -2225,7 +2144,7 @@ void cb_fullscreen_destoyed (GtkObject *o, Application *app) {
         app->fullscreen = false;
         app->window_fullscreen = NULL;
         view_area->controls = NULL;
-        view_area->controlsVisible (false);
+        view_area->controlsVisible (app, false);
     }
 }
 
