@@ -23,6 +23,7 @@
 #include <gtk/gtkwidget.h>
 
 #include "kmplayertypes.h"
+#include "surface.h"
 
 typedef struct _Application Application;
 
@@ -56,7 +57,7 @@ public:
     void mouseDoubleClickEvent ();
     void paintEvent (const IRect &);
     bool timeoutPaintEvent ();
-    void controlsVisible (bool show);
+    void controlsVisible (Application *app, bool show);
     KDE_NO_EXPORT Rect videoGeometry () const { return m_av_geometry; }
     int width () const;
     int height () const;
@@ -64,11 +65,13 @@ public:
     GdkWindow * videoWindow () const { return video; }
     Mrl *videoMrl () { return video_node ? video_node->mrl () : NULL; }
     void setOutputWidget (Application *, GtkWidget * w);
+    void setControlsOverlap (bool b) { m_controls_overlap = b; }
     GdkWindow *controls;
     int controls_timer;
 private:
     void syncVisual (const IRect & rect);
     void updateSurfaceBounds ();
+    bool scheduleRepaintTimeout (unsigned int cur_time);
     ViewAreaPrivate *d;
     RepaintUpdater *m_updaters;
     Rect m_av_geometry;
@@ -78,10 +81,15 @@ private:
     GdkWindow * video;
     SurfacePtr surface;
     NodePtrW video_node;
+    unsigned int last_repaint_time;
+    unsigned int next_repaint_time;
+    unsigned int penalty;
+    int cur_timeout;
     int m_repaint_timer;
     bool m_fullscreen;
     bool m_event_filter_on;
     bool m_updaters_enabled;
+    bool m_controls_overlap;
 };
 
 } // namespace KMPlayer
